#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['confuse']

package_data = \
{'': ['*']}

install_requires = \
['pyyaml']

extras_require = \
{"test:python_version == '2.7'": ['pathlib']}

setup(name='confuse',
      version='1.7.0',
      description='Painless YAML configuration.',
      author='Adrian Sampson',
      author_email='adrian@radbox.org',
      url='https://github.com/beetbox/confuse',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, <4',
     )
